@@ -266,65 +266,54 @@ desktop_entry_load_directory (DesktopEntry  *entry,
 static DesktopEntryResultCode
 desktop_entry_load (DesktopEntry *entry)
 {
-  DesktopEntryResultCode rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
-
   if (strstr (entry->path, "/menu-xdg/"))
-    return rescode;
-
+    return DESKTOP_ENTRY_LOAD_FAIL_OTHER;
   if (entry->type == DESKTOP_ENTRY_DESKTOP)
     {
       GKeyFile *key_file = NULL;
       DesktopEntryDesktop *entry_desktop = (DesktopEntryDesktop*)entry;
+      const char *categories_str;
+
+      entry_desktop->appinfo = g_desktop_app_info_new_from_filename (entry->path);
+      if (!entry_desktop->appinfo ||
+          !g_app_info_get_name (G_APP_INFO (entry_desktop->appinfo)) ||
+          !g_app_info_get_executable (G_APP_INFO (entry_desktop->appinfo)))
+        {
+          menu_verbose ("Failed to load \"%s\"\n", entry->path);
+          return DESKTOP_ENTRY_LOAD_FAIL_APPINFO;
+        }
+
+      categories_str = g_desktop_app_info_get_categories (entry_desktop->appinfo);
+      if (categories_str)
+        {
+          char **categories;
+          int i;
+
+          categories = g_strsplit (categories_str, ";", -1);
+          entry_desktop->categories = g_new0 (GQuark, g_strv_length (categories) + 1);
+
+          for (i = 0; categories[i]; i++)
+            entry_desktop->categories[i] = g_quark_from_string (categories[i]);
+
+          g_strfreev (categories);
+        }
 
       key_file = g_key_file_new ();
 
-      if (g_key_file_load_from_file (key_file, entry->path, 0, NULL))
-        {
-          entry_desktop->appinfo = g_desktop_app_info_new_from_keyfile (key_file);
-
-          if (!entry_desktop->appinfo ||
-              !g_app_info_get_name (G_APP_INFO (entry_desktop->appinfo)) ||
-              !g_app_info_get_executable (G_APP_INFO (entry_desktop->appinfo)))
-            {
-              menu_verbose ("Failed to load appinfo for \"%s\"\n", entry->path);
-              rescode = DESKTOP_ENTRY_LOAD_FAIL_APPINFO;
-            }
-          else
-            {
-              const char *categories_str;
-              categories_str = g_desktop_app_info_get_categories (entry_desktop->appinfo);
-
-              if (categories_str)
-                {
-                  char **categories;
-                  int i;
-
-                  categories = g_strsplit (categories_str, ";", -1);
-                  entry_desktop->categories = g_new0 (GQuark, g_strv_length (categories) + 1);
-
-                  for (i = 0; categories[i]; i++)
-                    entry_desktop->categories[i] = g_quark_from_string (categories[i]);
-
-                  g_strfreev (categories);
-                }
-
-              entry_desktop->showin = key_file_get_show_in (key_file);
-
-              rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
-            }
-        }
+      if (!g_key_file_load_from_file (key_file, entry->path, 0, NULL))
+        entry_desktop->showin = TRUE;
       else
-        {
-          menu_verbose ("Failed to read contents of \"%s\"\n", entry->path);
-          rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
-        }
+        entry_desktop->showin = key_file_get_show_in (key_file);
+
       g_key_file_free (key_file);
+
+      return DESKTOP_ENTRY_LOAD_SUCCESS;
     }
   else if (entry->type == DESKTOP_ENTRY_DIRECTORY)
     {
       GKeyFile *key_file = NULL;
       GError   *error = NULL;
-      rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
+      DesktopEntryResultCode rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
 
       key_file = g_key_file_new ();
 
@@ -355,18 +344,13 @@ desktop_entry_load (DesktopEntry *entry)
           else
             menu_verbose ("Failed to load \"%s\"\n", entry->path);
         }
+
+      return rescode;
     }
   else
     g_assert_not_reached ();
 
-  return rescode;
-}
-
-static gboolean
-code_failed (DesktopEntryResultCode code)
-{
-    return code == DESKTOP_ENTRY_LOAD_FAIL_OTHER ||
-           code == DESKTOP_ENTRY_LOAD_FAIL_APPINFO;
+  return DESKTOP_ENTRY_LOAD_FAIL_OTHER;
 }
 
 DesktopEntry *
@@ -405,7 +389,7 @@ desktop_entry_new (const char             *path,
   code = desktop_entry_load (retval);
   *res_code = code;
 
-  if (code_failed (code))
+  if (code < DESKTOP_ENTRY_LOAD_SUCCESS)
     {
       desktop_entry_unref (retval);
       return NULL;
@@ -447,7 +431,7 @@ desktop_entry_reload (DesktopEntry *entry)
   else
     g_assert_not_reached ();
 
-  if (code_failed (desktop_entry_load (entry)))
+  if (desktop_entry_load (entry) < DESKTOP_ENTRY_LOAD_SUCCESS)
     {
       desktop_entry_unref (entry);
       return NULL;
@@ -462,7 +446,7 @@ desktop_entry_ref (DesktopEntry *entry)
   g_return_val_if_fail (entry != NULL, NULL);
   g_return_val_if_fail (entry->refcount > 0, NULL);
 
-  g_atomic_int_inc (&entry->refcount);
+  entry->refcount += 1;
 
   return entry;
 }
@@ -696,43 +680,6 @@ desktop_entry_has_category (DesktopEntry *entry,
   return FALSE;
 }
 
-void
-desktop_entry_add_legacy_category (DesktopEntry *entry)
-{
-  GQuark *categories;
-  int     i;
-  DesktopEntryDesktop *desktop_entry;
-
-  g_return_if_fail (entry->type == DESKTOP_ENTRY_DESKTOP);
-
-  desktop_entry = (DesktopEntryDesktop*) entry;
-
-  menu_verbose ("Adding Legacy category to \"%s\"\n",
-                entry->basename);
-
-  if (desktop_entry->categories != NULL)
-    {
-      i = 0;
-      for (; desktop_entry->categories[i]; i++);
-
-      categories = g_new0 (GQuark, i + 2);
-
-      i = 0;
-      for (; desktop_entry->categories[i]; i++)
-        categories[i] = desktop_entry->categories[i];
-    }
-  else
-    {
-      categories = g_new0 (GQuark, 2);
-      i = 0;
-    }
-
-  categories[i] = g_quark_from_string ("Legacy");
-
-  g_free (desktop_entry->categories);
-  desktop_entry->categories = categories;
-}
-
 /*
  * Entry sets
  */
@@ -756,7 +703,7 @@ desktop_entry_set_ref (DesktopEntrySet *set)
   g_return_val_if_fail (set != NULL, NULL);
   g_return_val_if_fail (set->refcount > 0, NULL);
 
-  g_atomic_int_inc (&set->refcount);
+  set->refcount += 1;
 
   return set;
 }
@@ -764,13 +711,11 @@ desktop_entry_set_ref (DesktopEntrySet *set)
 void
 desktop_entry_set_unref (DesktopEntrySet *set)
 {
-  gboolean is_zero;
-
   g_return_if_fail (set != NULL);
   g_return_if_fail (set->refcount > 0);
 
-  is_zero = g_atomic_int_dec_and_test (&set->refcount);
-  if (is_zero)
+  set->refcount -= 1;
+  if (set->refcount == 0)
     {
       menu_verbose (" Deleting entry set %p\n", set);
 
